/* misc.h
   function headers for functions in misc.c */

     /*---------------------------------------------------------------*/
     /* Xgopher        version 1.1     20 April 1991                  */
     /*                version 1.0     04 March 1991                  */
     /* X window system client for the University of Minnesota        */
     /*                                Internet Gopher System.        */
     /* Allan Tuchman, University of Illinois at Urbana-Champaign     */
     /*                Computing Services Office                      */
     /* Copyright 1992 by                                             */
     /*           the Board of Trustees of the University of Illinois */
     /* Permission is granted to freely copy and redistribute this    */
     /* software with the copyright notice intact.                    */
     /*---------------------------------------------------------------*/

#include "gopher.h"

gopherTime timeNow(
);

gopherTime timeSince(
#ifdef PROTO
	gopherTime
#endif
);

void setDirTime(
#ifdef PROTO
	gopherDirP
#endif
);

void clearDirWhenOld(
#ifdef PROTO
	gopherDirP
#endif
);

char    **makeWordList(
#ifdef PROTO
	char    *
#endif
);

void freeWordList(
#ifdef PROTO
	char **
#endif
);

char	*cmdPath(
#ifdef PROTO
	char	*
#endif
);

void vStringSet(
#ifdef PROTO
	vString *,	/* vs */
	char *		/* value */
#endif
);
