/* listP.h
   Program options used in list processing routines */

     /*---------------------------------------------------------------*/
     /* Xgopher        version 1.1     20 April 1991                  */
     /*                version 1.0     04 March 1991                  */
     /* X window system client for the University of Minnesota        */
     /*                                Internet Gopher System.        */
     /* Allan Tuchman, University of Illinois at Urbana-Champaign     */
     /*                Computing Services Office                      */
     /* Copyright 1992 by                                             */
     /*           the Board of Trustees of the University of Illinois */
     /* Permission is granted to freely copy and redistribute this    */
     /* software with the copyright notice intact.                    */
     /*---------------------------------------------------------------*/


	/* initial number of gopher items to allocate, and increment if
	   xgopher runs out of items. */

int	itemStart, itemIncrement;

	/* initial number of gopher directories to allocate, and increment if
	   xgopher runs out of directories. */

int	dirStart,  dirIncrement;
